@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 Zope Corporation and Contributors.
+# Copyright (c) 2004 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """Session implementation
-
-$Id: session.py 90600 2008-08-29 21:32:29Z fdrake $
 """
 from cStringIO import StringIO
 import time, string, random, thread
@@ -31,7 +29,6 @@ import zope.component
 import zope.interface
 from zope.component.interfaces import ComponentLookupError
 from zope.publisher.interfaces import IRequest
-from zope.annotation.interfaces import IAttributeAnnotatable
 
 from zope.session.interfaces import \
         IClientIdManager, IClientId, ISession, ISessionDataContainer, \
@@ -74,9 +71,10 @@ class PersistentSessionDataContainer(zope.location.Location,
     IterableUserDict):
     """A SessionDataContainer that stores data in the ZODB"""
 
-    zope.interface.implements(ISessionDataContainer, IAttributeAnnotatable)
+    zope.interface.implements(ISessionDataContainer)
 
     _v_last_sweep = 0 # Epoch time sweep last run
+    disable_implicit_sweeps = False
 
     def __init__(self):
         self.data = OOBTree()
@@ -106,6 +104,35 @@ class PersistentSessionDataContainer(zope.location.Location,
                 [...]
             KeyError: 'clientid'
 
+        Can you disable the automatic removal of stale data.
+
+            >>> sdc.disable_implicit_sweeps = True
+            >>> sdc['stale'] = stale = SessionData()
+
+        Now we try the same method of winding back the clock.
+
+            >>> stale.setLastAccessTime(sd.getLastAccessTime() - 64)
+            >>> sdc._v_last_sweep = sdc._v_last_sweep - 4
+
+        But the data is not automatically removed.
+
+            >>> sdc['stale'] #doctest: +ELLIPSIS
+            <zope.session.session.SessionData object at ...>
+
+        We can manually remove stale data by calling sweep() if stale
+        data isn't being automatically removed.
+
+            >>> stale.setLastAccessTime(sd.getLastAccessTime() - 64)
+            >>> sdc.sweep()
+            >>> sdc['stale']
+            Traceback (most recent call last):
+                [...]
+            KeyError: 'stale'
+
+        Now we turn automatic removal back on.
+
+            >>> sdc.disable_implicit_sweeps = False
+
         Ensure the lastAccessTime on the ISessionData is being updated
         occasionally. The ISessionDataContainer maintains this whenever
         the ISessionData is set or retrieved.
@@ -205,7 +232,8 @@ class PersistentSessionDataContainer(zope.location.Location,
             except AttributeError:
                 pass
 
-        if self._v_last_sweep + self.resolution < now:
+        if (self._v_last_sweep + self.resolution < now and
+            not self.disable_implicit_sweeps):
             self.sweep()
             if getattr(self, '_v_old_sweep', None) is None:
                 self._v_old_sweep = self._v_last_sweep
